<?php

namespace Drupal\graphql_vertex_ai\GraphQL;

use GraphQL\Language\AST\Node;

/**
 * Navigate the AST\Node architecture to get useful information.
 */
class ASTNodeQuery {

  public function __construct(protected Node $node) {}

  /**
   * Search the AST node hierarchy for a field.
   *
   * @param string $path
   *   The '.' separated path to search for.
   *
   * @return bool
   *   Whether we found the path or not.
   */
  public function hasField(string $path): bool {
    $node = $this->nodeAtPath($path);
    return !empty($node);
  }

  /**
   * Get the value of the named parameter at the node path.
   *
   * @param string $path
   *   The path of the node.
   * @param string $parameter
   *   The name of the parameter.
   *
   * @return string|null
   *   The value if found.
   */
  public function getParameter(string $path, string $parameter): ?string {
    $node = $this->nodeAtPath($path);
    if (!$node) {
      return NULL;
    }
    foreach ($node->arguments as $argument) {
      if ($argument->name->value === $parameter) {
        return $argument->value->value;
      }
    }
    return NULL;
  }

  /**
   * The AST Node at the given path.
   *
   * @param string $path
   *   The path of the node.
   *
   * @return \GraphQL\Language\AST\Node|null
   *   The node if found.
   */
  private function nodeAtPath(string $path): ?Node {
    $fields = explode('.', $path);
    return $this->doNodeAtPath($this->node->selectionSet->selections, $fields);
  }

  /**
   * Navigate the tree recursively looking for a node path.
   *
   * @param \GraphQL\Language\AST\Node[] $nodeList
   *   The next node in the tree.
   * @param array $fields
   *   The exploded path to search for.
   *
   * @return \GraphQL\Language\AST\Node|null
   *   The node if we found it.
   */
  private function doNodeAtPath(iterable $nodeList, array $fields): ?Node {
    $field = array_shift($fields);
    /** @var \GraphQL\Language\AST\FieldNode $node */
    foreach ($nodeList as $node) {
      if ($node->name->value === $field) {
        if (empty($fields)) {
          return $node;
        }
        $children = $node->selectionSet?->selections ?? [];
        return $this->doNodeAtPath($children, $fields);
      }
    }
    return NULL;
  }

}
