<?php

namespace Drupal\graphql_vertex_ai\Plugin\GraphQL\DataProducer;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\graphql\GraphQL\Execution\FieldContext;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Drupal\graphql_vertex_ai\GraphQL\ASTNodeQuery;
use Drupal\key\KeyRepositoryInterface;
use Google\Cloud\DiscoveryEngine\V1\Client\SearchServiceClient;
use Google\Cloud\DiscoveryEngine\V1\SearchRequest;
use Google\Cloud\DiscoveryEngine\V1\SearchRequest\ContentSearchSpec;
use Google\Cloud\DiscoveryEngine\V1\SearchRequest\ContentSearchSpec\SnippetSpec;
use Google\Cloud\DiscoveryEngine\V1\SearchRequest\ContentSearchSpec\SummarySpec;
use Google\Cloud\DiscoveryEngine\V1\SearchRequest\FacetSpec;
use Google\Cloud\DiscoveryEngine\V1\SearchRequest\FacetSpec\FacetKey;
use Google\Cloud\DiscoveryEngine\V1\SearchResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The data producer for search queries.
 *
 * @DataProducer(
 *   id = "vertex_ai_search_query",
 *   name = @Translation("Search query"),
 *   description = @Translation("Search query"),
 *   produces = @ContextDefinition("any",
 *     label = @Translation("Search result")
 *   ),
 *   consumes = {
 *     "query" = @ContextDefinition("string",
 *       label = @Translation("Query"),
 *       required = TRUE
 *     ),
 *     "pageSize" = @ContextDefinition("integer",
 *       label = @Translation("Page Size"),
 *       required = TRUE
 *     ),
 *     "offset" = @ContextDefinition("integer",
 *       label = @Translation("Offset"),
 *       required = TRUE
 *     ),
 *     "filter" = @ContextDefinition("string",
 *       label = @Translation("Filter"),
 *       required = FALSE
 *     ),
 *     "astContext" = @ContextDefinition("any",
 *        label = @Translation("AST Context"),
 *        required = TRUE
 *      ),
 *   }
 * )
 */
class VertexAiSearchQuery extends DataProducerPluginBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    /** @var \Drupal\Core\Config\ConfigFactoryInterface $config_factory */
    $config_factory = $container->get('config.factory');
    /** @var \Drupal\key\KeyRepositoryInterface $key_repository */
    $key_repository = $container->get('key.repository');
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $config_factory,
      $key_repository,
    );
  }

  /**
   * VertexAiSearchQuery constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory service.
   * @param \Drupal\key\KeyRepositoryInterface $keyRepository
   *   The key repository service.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    array $plugin_definition,
    protected ConfigFactoryInterface $configFactory,
    protected KeyRepositoryInterface $keyRepository,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * Resolver.
   *
   * @param string $query
   *   The search query.
   * @param int $pageSize
   *   The number of results per page.
   * @param int $offset
   *   The offset for the page results.
   * @param string $filter
   *   A filter for schema metadata to apply to the query.
   * @param \Drupal\graphql_vertex_ai\GraphQL\ASTNodeQuery $nodeQuery
   *   The document model.
   * @param \Drupal\graphql\GraphQL\Execution\FieldContext $metadata
   *   The metadata.
   *
   * @return \Google\Cloud\DiscoveryEngine\V1\SearchResponse
   *   The result.
   */
  public function resolve(string $query, int $pageSize, int $offset, ?string $filter, ASTNodeQuery $nodeQuery, FieldContext $metadata): SearchResponse {
    $config = $this->configFactory->get('graphql_vertex_ai.settings');
    $key = $this->keyRepository->getKey($config->get('key'))->getKeyValues();

    $searchServiceClient = new SearchServiceClient([
      'credentials' => $key,
    ]);

    $formattedServingConfig = SearchServiceClient::servingConfigName(
      $config->get('project_id'),
      $config->get('location'),
      $config->get('data_store_id'),
      $config->get('serving_config_id')
    );

    // Prepare the search request.
    $request = (new SearchRequest())->setServingConfig($formattedServingConfig);

    $request->setQuery($query);
    $request->setSafeSearch(TRUE);
    $request->setPageSize($pageSize);
    $request->setOffset($offset);
    if (!empty($filter)) {
      $request->setFilter($filter);
    }

    // Turn on facets.
    $facetParameter = $nodeQuery->getParameter('facets', 'keys');
    if (!empty($facetParameter)) {
      $facets = array_map('trim', explode(',', $facetParameter));
      foreach ($facets as $facet) {
        $key = (new FacetKey())->setKey($facet);
        $facetSpec = (new FacetSpec())->setFacetKey($key);
        $facetSpec->setLimit(50);
        $request->setFacetSpecs([$facetSpec]);
      }
    }

    $contentSearchSpec = new ContentSearchSpec();
    // Turn on summary.
    if ($nodeQuery->hasField('summary')) {
      $summarySpec = (new SummarySpec())->setSummaryResultCount(3);
      $contentSearchSpec = $contentSearchSpec->setSummarySpec($summarySpec);
    }
    // Turn on snippets.
    if ($nodeQuery->hasField('results.snippet')) {
      $snippetSpec = (new SnippetSpec())->setReturnSnippet(TRUE);
      $contentSearchSpec = $contentSearchSpec->setSnippetSpec($snippetSpec);
    }
    $request->setContentSearchSpec($contentSearchSpec);

    $response = $searchServiceClient->search($request);
    $results = $response->getPage()->getResponseObject();

    $metadata->mergeCacheMaxAge(0);

    return $results;
  }

}
