<?php

declare(strict_types=1);

namespace Drupal\graphql_vertex_ai\Plugin\GraphQL\SchemaExtension;

use Drupal\graphql\GraphQL\ResolverBuilder;
use Drupal\graphql\GraphQL\ResolverRegistryInterface;
use Drupal\graphql\Plugin\GraphQL\SchemaExtension\SdlSchemaExtensionPluginBase;
use Drupal\graphql_vertex_ai\GraphQL\Resolver\ASTContext;
use Google\Cloud\DiscoveryEngine\V1\SearchResponse;
use Google\Cloud\DiscoveryEngine\V1\SearchResponse\Facet;
use Google\Cloud\DiscoveryEngine\V1\SearchResponse\Facet\FacetValue;
use Google\Cloud\DiscoveryEngine\V1\SearchResponse\SearchResult;

/**
 * Schema extension for Vertex AI definitions.
 *
 * @SchemaExtension(
 *   id = "vertex_ai_search_query",
 *   name = "Vertex AI search query",
 *   description = "Schema extension that adds Vertex AI support.",
 *   schema = "composable"
 * )
 */
class SearchQuerySchemaExtension extends SdlSchemaExtensionPluginBase {

  /**
   * {@inheritdoc}
   */
  public function registerResolvers(ResolverRegistryInterface $registry): void {
    $builder = new ResolverBuilder();
    $this->addQueryFields($registry, $builder);
  }

  /**
   * Adds query fields.
   *
   * @param \Drupal\graphql\GraphQL\ResolverRegistryInterface $registry
   *   The registry to add fields to.
   * @param \Drupal\graphql\GraphQL\ResolverBuilder $builder
   *   The builder service.
   */
  protected function addQueryFields(ResolverRegistryInterface $registry, ResolverBuilder $builder): void {

    // Search response wrapper.
    $registry->addFieldResolver('VertexAiSearchResponse', 'totalResults',
      $builder->callback(fn(SearchResponse $response) => $response->getTotalSize())
    );
    $registry->addFieldResolver('VertexAiSearchResponse', 'summary',
      $builder->compose(
        $builder->produce('vertex_ai_search_response_summary_text')
          ->map('response', $builder->fromParent()),
        $builder->produce('vertex_ai_html_decode')
          ->map('text', $builder->fromParent())
      )
    );
    $registry->addFieldResolver('VertexAiSearchResponse', 'results',
      $builder->callback(fn (SearchResponse $response) => $response->getResults())
    );
    $registry->addFieldResolver('VertexAiSearchResponse', 'facets',
      $builder->callback(fn (SearchResponse $response) => $response->getFacets())
    );

    // Individual search results.
    $registry->addFieldResolver('VertexAiSearchResult', 'id',
      $builder->callback(fn(SearchResult $result) => $result->getId())
    );
    $registry->addFieldResolver('VertexAiSearchResult', 'name',
      $builder->callback(fn(SearchResult $result) => $result->getDocument()->getName())
    );
    $registry->addFieldResolver('VertexAiSearchResult', 'title',
      $builder->compose(
        $builder->produce('vertex_ai_search_result_derived_data')
          ->map('result', $builder->fromParent())
          ->map('field', $builder->fromValue('htmlTitle')),
        $builder->produce('vertex_ai_html_decode')
          ->map('text', $builder->fromParent())
      )
    );
    $registry->addFieldResolver('VertexAiSearchResult', 'uri',
      $builder->produce('vertex_ai_search_result_derived_data')
        ->map('result', $builder->fromParent())
        ->map('field', $builder->fromValue('link'))
    );
    $registry->addFieldResolver('VertexAiSearchResult', 'snippet',
      $builder->compose(
        $builder->produce('vertex_ai_search_result_derived_data')
          ->map('result', $builder->fromParent())
          ->map('field', $builder->fromValue('snippets.0.snippet')),
        $builder->produce('vertex_ai_html_decode')
          ->map('text', $builder->fromParent())
      )
    );

    // Facets.
    $registry->addFieldResolver('VertexAiFacet', 'key',
      $builder->callback(fn (Facet $facet) => $facet->getKey())
    );
    $registry->addFieldResolver('VertexAiFacet', 'values',
      $builder->callback(fn (Facet $facet) => $facet->getValues())
    );
    $registry->addFieldResolver('VertexAiFacetValue', 'value',
      $builder->callback(fn (FacetValue $facetValue) => $facetValue->getValue())
    );
    $registry->addFieldResolver('VertexAiFacetValue', 'count',
      $builder->callback(fn (FacetValue $facetValue) => $facetValue->getCount())
    );

    // Autocomplete.
    $registry->addFieldResolver('VertexAiAutocompleteResult', 'suggestions',
      $builder->produce('vertex_ai_autocomplete_response_suggestions')
        ->map('response', $builder->fromParent())
    );

    // The query.
    $model = new ASTContext();
    $registry->addFieldResolver('Query', 'vertexAiSearchQuery',
      $builder->produce('vertex_ai_search_query')
        ->map('query', $builder->fromArgument('query'))
        ->map('pageSize', $builder->fromArgument('pageSize'))
        ->map('offset', $builder->fromArgument('offset'))
        ->map('filter', $builder->fromArgument('filter'))
        ->map('astContext', $model)
    );

    // The autocomplete.
    $registry->addFieldResolver('Query', 'vertexAiAutocompleteQuery',
      $builder->produce('vertex_ai_autocomplete_query')
        ->map('query', $builder->fromArgument('query'))
    );
  }

}
